/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.db;

import com.tplan.robot.preferences.Preference;
import com.tplan.robot.scripting.ParametrizedTestScript;
import com.tplan.robot.scripting.ScriptEvent;
import com.tplan.robot.scripting.ScriptListener;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Properties;

/**
 * Java test script which creates a database connection.
 */
public class Connect extends AbstractDbScript implements ParametrizedTestScript, ScriptListener {

    // Reference to the created connection pool key. It is populated only when the
    // connection is created with pool=false.
    private String connectionPoolKey;

    @Override
    public void test() {

        // Read the database URL and the driver class name
        String url = getVariableAsString("url");
        if (url == null || url.trim().isEmpty()) {
            throw new IllegalStateException("The 'url' parameter is mandatory!");
        }

        String driver = getVariableAsString("driver");
        if (driver == null || driver.trim().isEmpty()) {
            throw new IllegalStateException("The 'driver' parameter is mandatory!");
        }

        // Create a key for the connection pool.
        StringBuilder keyBuf = new StringBuilder(url);
        keyBuf.append(';').append("driver=").append(driver);

        // Read the parameters and put them to the connection properties
        Properties connectionProps = new Properties();
        String s = getVariableAsString("user");
        if (s != null) {
            connectionProps.put("user", s);
            keyBuf.append(";user=").append(s);
        }
        s = getVariableAsString("password");
        if (s != null) {
            connectionProps.put("password", s);
        }
        Boolean autoCommit = getVariableAsBoolean("autocommit");
        Boolean makeFail = getVariableAsBoolean("makefail");

        // If pooling is deselected add the context hash code to the pool key
        // to make it visible just to the particular executing script. Also
        // register as listener to the script manager to get notified of the
        // execution end when we need to close the connection.
        Boolean pool = getVariableAsBoolean("pool");
        if (pool != null && !pool) {
            keyBuf.insert(0, Integer.toHexString(hashCode()) + "_");
            getContext().getScriptManager().addScriptListener(this);
        }

        // Remove the _DB_ERROR variable if it exists
        getContext().getVariables().remove(VAR_DB_ERROR);

        // Look in the pool first
        final String poolKey = keyBuf.toString();
        Connection conn = null;
        synchronized (connectionPool) {
            ConnectionBean bean = connectionPool.get(poolKey);
            if (bean != null) {
                conn = bean.connection;
                try {
                    if (conn.isClosed()) {
                        // The connection is closed -> remove it
                        connectionPool.remove(poolKey);
                    } else {
                        // The connection is active -> reuse it
                        setCurrentConnection(bean, poolKey, autoCommit, makeFail, false);
                        return;
                    }
                } catch (SQLException ex) {
                    connectionPool.remove(poolKey);
                }
            }
        }


        // Instantiate the driver
        try {
            Class.forName(driver);
        } catch (Throwable t) {
            String msg = "Failed to load the '" + driver + "' driver: " + t.getMessage() + ". Make sure "
                    + "that the driver class or JAR is on the Java class path or that you load it using Include "
                    + "prior to calling this command.";
            fail(msg, 1);
            return;
        }

        try {
            // Create the connection and set it as the current one
            conn = DriverManager.getConnection(url, connectionProps);
            setCurrentConnection(new ConnectionBean(conn), poolKey, autoCommit, makeFail, true);

            // Register the shut down hook
            DBShutdownHook.register();

            // If pooling is off save a reference so that we can close it after 
            // the test script finishes
            if (pool != null && !pool) {
                connectionPoolKey = poolKey;
            }

        } catch (SQLException ex) {
            String msg = "Failed to open connection to the database at " + url + ": " + ex.getMessage();
            fail(msg, 2);
            return;
        }
        setVariable("_EXIT_CODE", "0");
    }

    public List<String> getParameters() {
        List<String> l = new ArrayList<String>(5);
        l.add("url");
        l.add("driver");
        l.add("user");
        l.add("password");
        l.add("autocommit");
        l.add("pool");
        l.add("makefail");
        return l;
    }

    public List getParameterValues(String parameterName) {
        return null;
    }

    public void setParameters(Map<String, String> paramsAndValues) {
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>(4);
        String title = "Connection parameters";
        Preference p = new Preference("url", Preference.TYPE_STRING, "Connection URL", null);
        p.setMandatory(true);
        p.setAcceptEmptyValue(false);
        p.setPreferredContainerName(title);
        l.add(p);

        p = new Preference("driver", Preference.TYPE_STRING, "JDBC driver class", null);
        p.setPreferredContainerName(title);
        p.setAcceptEmptyValue(false);
        p.setMandatory(true);
        l.add(p);

        p = new Preference("user", Preference.TYPE_STRING, "User name", null);
        p.setAcceptEmptyValue(false);
        p.setPreferredContainerName(title);
        l.add(p);

        p = new Preference("password", Preference.TYPE_STRING, "Password", null);
        p.setAcceptEmptyValue(false);
        p.setPreferredContainerName(title);
        l.add(p);

        p = new Preference("autocommit", Preference.TYPE_BOOLEAN, "Auto commit mode", null);
        p.setHint("Auto Commit Mode",
                "<html><body>When a connection is created, it is by default in the auto-commit mode. This means that each "
                + "individual SQL statement is treated as a transaction and is automatically "
                + "committed right after it is executed. To override this behavior set this parameter to <code>false</code> "
                + "and commit the transactions by calls of the <b>Commit</b> test script.</body></html>");
        p.setAcceptEmptyValue(false);
        p.setDefaultValue(true);
        p.setPreferredContainerName(title);
        l.add(p);

        p = new Preference("pool", Preference.TYPE_BOOLEAN, "Pool the connection", null);
        p.setAcceptEmptyValue(false);
        p.setDefaultValue(true);
        p.setPreferredContainerName(title);
        l.add(p);

        p = new Preference("makefail", Preference.TYPE_BOOLEAN, "Terminate the calling script on any DB error", null);
        p.setAcceptEmptyValue(false);
        p.setDefaultValue(true);
        p.setPreferredContainerName(title);
        l.add(p);
        return l;
    }

    public String getShortDescription() {
        return "Connect to a SQL database over JDBC";
    }

    public String getLongDescription() {
        return getShortDescription();
    }

    public void scriptEvent(ScriptEvent event) {
        if (event.getType() == ScriptEvent.SCRIPT_EXECUTION_FINISHED) {
            // Remove this class from the listener list
            event.getScriptManager().removeScriptListener(this);

            // Remove the connection from the pool and close it.
            if (connectionPoolKey != null) {
                ConnectionBean bean = connectionPool.remove(connectionPoolKey);

                // Close the connection
                if (bean != null) {
                    try {
                        bean.connection.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }
}
