/*
 * T-Plan Robot Enterprise, automated testing tool.
 * Copyright (C) 2009-2013 T-Plan Limited (http://www.t-plan.com),
 * Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
 * All rights reserved.
 */
package com.tplan.db;

import com.tplan.robot.preferences.Preference;
import com.tplan.robot.scripting.ParametrizedTestScript;
import java.io.IOException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

/**
 * Java test script which executes a SQL query on the current connection.
 */
public class Execute extends AbstractDbScript implements ParametrizedTestScript {

    public static final String VAR_DB_COLUMN_COUNT = "_DB_COLUMN_COUNT";
    public static final String VAR_DB_ROW_COUNT = "_DB_ROW_COUNT";
    public static final String VAR_DB_VALUE = "_DB_VALUE_{0}_{1}";
    private static final String PARAM_COMMAND = "statement";

    @Override
    public void test() {
        // Get the statement
        String cmd = getVariableAsString(PARAM_COMMAND);
        if (cmd == null || cmd.isEmpty()) {
            throw new IllegalStateException("The '" + PARAM_COMMAND + "' parameter is mandatory!");
        }

        // Get the current connection
        Connection conn = getCurrentConnection(true);
        if (conn == null) {
            return;
        }

        // Clean up the variables
        Integer rows = getVariableAsInt(VAR_DB_ROW_COUNT);
        Integer columns = getVariableAsInt(VAR_DB_COLUMN_COUNT);
        final Map variables = getContext().getVariables();

        if (rows != null && columns != null) {
            String var;
            for (int row = 0; row < rows; row++) {
                for (int column = 0; column < columns; column++) {
                    var = MessageFormat.format(VAR_DB_VALUE, row, column);
                    variables.remove(var);
                }
            }
            variables.remove(VAR_DB_ROW_COUNT);
            variables.remove(VAR_DB_COLUMN_COUNT);
        }

        variables.remove(VAR_DB_ERROR);

        // Execute the query
        Statement stmt = null;

        try {
            stmt = conn.createStatement();

            // Parse the SQL command. It will tell us whether we should call
            // Statement.executeQuery() or Statement.executeUpdate()
            Scanner scanner = new Scanner(cmd);
            String commandName = scanner.next();

            if (!commandName.equalsIgnoreCase("select")) {
                // An update command
                stmt.executeUpdate(cmd);
            } else { // A select command

                // If the query
                ResultSet rs = stmt.executeQuery(cmd);

                // Iterate over the rows and store the values to the variable table
                // in the script context
                int row = 1, col;
                int columnCount = rs.getMetaData().getColumnCount();
                String var;
                Object value;

                while (rs.next()) {
                    for (col = 1; col <= columnCount; col++) {
                        var = MessageFormat.format(VAR_DB_VALUE, row, col);
                        value = rs.getObject(col);
                        variables.put(var, value == null ? "" : value.toString());
                    }
                    row++;
                }
                variables.put(VAR_DB_ROW_COUNT, Integer.toString(row - 1));
                variables.put(VAR_DB_COLUMN_COUNT, Integer.toString(columnCount));
            }
        } catch (SQLException e) {
            String msg = "The statement failed to execute: " + e.getMessage();
            fail(msg, 2);
            return;
        } finally {
            if (stmt != null) {
                try {
                    stmt.close();
                } catch (SQLException se) {
                    se.printStackTrace();
                }
            }
        }
        setVariable("_EXIT_CODE", "0");
    }

    public List<String> getParameters() {
        List<String> l = new ArrayList<String>(2);
        l.add(PARAM_COMMAND);
        return l;
    }

    public List getParameterValues(String parameterName) {
        return null;
    }

    public void setParameters(Map<String, String> paramsAndValues) {
    }

    public List<Preference> getVisualParameters() {
        List<Preference> l = new ArrayList<Preference>(4);
        String title = "Execution parameters";
        Preference p = new Preference(PARAM_COMMAND, Preference.TYPE_STRING, "SQL statement", null);
        p.setMandatory(true);
        p.setAcceptEmptyValue(false);
        p.setPreferredContainerName(title);
        l.add(p);
        return l;
    }

    public String getShortDescription() {
        return "Execute a SQL query on the current database connection.";
    }

    public String getLongDescription() {
        return getShortDescription();
    }
}
